// ----------------------------------------------------------
// This Source Code Form is subject to the terms of the
// Mozilla Public License, v.2.0. If a copy of the MPL
// was not distributed with this file, You can obtain one
// at http://mozilla.org/MPL/2.0/.
// ----------------------------------------------------------
// Codebase: https://github.com/ArKuznetsov/iracli/
// ----------------------------------------------------------

Перем ПараметрыПодключения;                // - Структура - прочитанные параметры подключения к
                                           //               сервису администрирования и авторизации кластеров и ИБ

Перем ПараметрыСчетчиков;                  // - Структура   - настройки счетчиков по типам объектов
                                           //                 (см. ПараметрыСчетчиков())

Перем МаксПопытокИнициализацииКластера;    // - Число     - количество попыток инициализации кластера 1С
                                           //               обход проблемы с пусты выводом команды `rac cluster list`
Перем ИнтервалПовторнойИнициализации;      // - Число     - задержка перед повторным подключением (мсек.)
                                           //               умножается на номер попытки подключения

Перем ЛогироватьЗамерыВремени;             // - Булево    - Истина - будет выполняться логирование
                                           //               времени выполнения запросов в файл
Перем ФайлЛогаЗамеровВремени;              // - Строка    - путь к файлу лога замеров времени

#Область ПрограммныйИнтерфейс

// Процедура - инициализирует модуль и читает переданные настройки или настройки из файла "racsettings.json"
//
// Параметры:
//   НовыеНастройкиПодключения   - Соответствие  - настройки из которых будут заполняться параметры
//
Процедура Инициализация(Знач НовыеНастройкиПодключения = Неопределено) Экспорт

	Если ТипЗнч(НовыеНастройкиПодключения) = Тип("Структура")
	 ИЛИ ТипЗнч(НовыеНастройкиПодключения) = Тип("Соответствие") Тогда
		НастройкиДляУстановки = НовыеНастройкиПодключения;
	ИначеЕсли ТипЗнч(НовыеНастройкиПодключения) = Тип("Строка") Тогда
		НастройкиДляУстановки = ОбщегоНазначения.ПрочитатьДанныеИзМакетаJSON(НовыеНастройкиПодключения, Истина);
	Иначе
		НастройкиДляУстановки = ОбщегоНазначения.ПрочитатьДанныеИзМакетаJSON("/config/racsettings", Истина);
	КонецЕсли;
	
	ЗаполнитьПараметрыПодключения(НастройкиДляУстановки);
	
	ЗаполнитьПараметры(НастройкиДляУстановки);

	УстановитьПараметрыСчетчиков();

КонецПроцедуры // Инициализация()

// Функция - возвращает параметры подключения к сервису администрирования
// и авторизации кластеров и ИБ
//
// Возвращаемое значение:
//   Структура               - параметры подключения
//      *Агенты    - Структура   - параметры подключения к сервисам аднистрирования 1С
//      *Кластеры  - Структура   - параметры авторизации кластеров и ИБ
//
Функция ПараметрыПодключения() Экспорт

	Возврат ПараметрыПодключения;

КонецФункции // ПараметрыПодключения()

// Функция - возвращает структуру параметров счетчиков
// (см. ../../config/counters.json)
//
// Возвращаемое значение:
//   Соответствие     - структура параметров счетчиков
//     * <тип объектов>            - Строка                     - параметры счетчиков для объектов указанного типа
//       ** counter_prefix            - Строка                     - текстовый префикс счетчиков
//       ** dimentions                - Соответствие               - доступные измерения счетчиков
//         *** <имя измерения>          - Соответствие               - описание доступного измерения счетчиков
//           **** name                    - Строка                     - имя измерения
//           **** name_rac                - Строка                     - имя, как оно возвращается утилитой RAC
//           **** description             - Строка                     - текстовое описание измерения
//       ** counters                  - Соответствие               - описание доступных счетчиков
//         *** <имя счетчика>           - Соответствие               - описание доступного счетчика
//           **** use                     - Булево                     - флаг использования счетчика
//           **** name                    - Строка                     - имя счетчика
//           **** description             - Булево                     - текстовое описание счетчика
//           **** dimentions              - Массив из Строка           - имена измерений счетчика из доступных
//           **** aggregate               - Строка                     - агрегатная функция (см. АгрегатныеФункции())
//
Функция ПараметрыСчетчиков() Экспорт
	
	Возврат ПараметрыСчетчиков;

КонецФункции // ПараметрыСчетчиков()

// Функция - возвращает количество попыток инициализации кластера 1С
// обход проблемы с пусты выводом команды `rac cluster list`
//
// Возвращаемое значение:
//   Число     - количество попыток инициализации кластера 1С
//
Функция МаксПопытокИнициализацииКластера() Экспорт

	Возврат МаксПопытокИнициализацииКластера;

КонецФункции // МаксПопытокИнициализацииКластера()

// Функция - возвращает задержку перед повторным подключением к кластеру 1С (мсек.)
// умножается на номер попытки подключения
//
// Возвращаемое значение:
//   Число     - задержка перед повторным подключением (мсек.)
//
Функция ИнтервалПовторнойИнициализации() Экспорт

	Возврат ИнтервалПовторнойИнициализации;

КонецФункции // ИнтервалПовторнойИнициализации()

// Функция - возвращает флаг логирования замеров времени
//
// Возвращаемое значение:
//   Булево    - Истина - будет выполняться логирование
//                        времени выполнения запросов в файл
//
Функция ЛогироватьЗамерыВремени() Экспорт

	Возврат ЛогироватьЗамерыВремени;

КонецФункции // ЛогироватьЗамерыВрени()

// Функция - возвращает путь к файлу лога замеров времени
//
// Возвращаемое значение:
//   Строка    - путь к файлу лога замеров времени
//
Функция ФайлЛогаЗамеровВремени() Экспорт

	Возврат ФайлЛогаЗамеровВремени;

КонецФункции // ФайлЛогаЗамеровВремени()

#КонецОбласти // ПрограммныйИнтерфейс

#Область ЧтениеНастроек

// Процедура - заполняет параметры подключения к сервису администрирования 
// и авторизации кластеров из переданных настроек
//
// Параметры:
//   НастройкиДляУстановки   - Соответствие  - настройки из которых будут заполняться параметры
//
Процедура ЗаполнитьПараметрыПодключения(НастройкиДляУстановки)

	НастройкиАгентов   = ПолучитьЗначениеНастройки("ras"    , НастройкиДляУстановки);
	НастройкиКластеров = ПолучитьЗначениеНастройки("cluster", НастройкиДляУстановки);

	ПараметрыАгентов = Новый Соответствие();
	ИдентификаторыАгентов = Новый Массив();

	Для Каждого ТекНастройки Из НастройкиАгентов Цикл
	
		Если ВРег(ТекНастройки.Ключ) = "__DEFAULT" И НЕ НастройкиАгентов.Количество() = 1 Тогда
			Продолжить;
		КонецЕсли;

		ПараметрыАгента = ПараметрыАгента(ТекНастройки.Ключ, НастройкиАгентов);

		ПараметрыАгентов.Вставить(ВРег(ТекНастройки.Ключ), ПараметрыАгента);

		ИдентификаторыАгентов.Добавить(ВРег(ТекНастройки.Ключ));

	КонецЦикла;

	ПараметрыКластеров = Новый Соответствие();

	Для Каждого ТекНастройки Из НастройкиКластеров Цикл
	
		ПараметрыКластера = ПараметрыКластера(ТекНастройки.Ключ, НастройкиКластеров);

		Если ВРег(ТекНастройки.Ключ) = "__DEFAULT" Тогда
			КлючНастроек = "ПОУМОЛЧАНИЮ";
		Иначе
			КлючНастроек = ВРег(ТекНастройки.Ключ);
		КонецЕсли;

		ПараметрыКластеров.Вставить(КлючНастроек, ПараметрыКластера);

	КонецЦикла;

	ПараметрыПодключения = Новый Структура();
	ПараметрыПодключения.Вставить("Агенты"               , ПараметрыАгентов);
	ПараметрыПодключения.Вставить("ИдентификаторыАгентов", ИдентификаторыАгентов);
	ПараметрыПодключения.Вставить("Кластеры"             , ПараметрыКластеров);

КонецПроцедуры // ЗаполнитьПараметрыПодключения()

// Процедура - заполняет параметры подключения к сервису администрирования 
// и авторизации кластеров из переданных настроек
//
// Параметры:
//   НастройкиДляУстановки   - Соответствие  - настройки из которых будут заполняться параметры
//
Процедура ЗаполнитьПараметры(НастройкиДляУстановки)

	МаксПопытокИнициализацииКластера = ПолучитьЗначениеНастройки("reconnectAtempts", НастройкиДляУстановки);
	ИнтервалПовторнойИнициализации   = ПолучитьЗначениеНастройки("reconnectInterval", НастройкиДляУстановки);
	ЛогироватьЗамерыВремени          = ПолучитьЗначениеНастройки("logQueryDuration", НастройкиДляУстановки);
	ФайлЛогаЗамеровВремени           = ПолучитьЗначениеНастройки("queryDurationLogFilename", НастройкиДляУстановки);

	Если МаксПопытокИнициализацииКластера = Неопределено Тогда
		МаксПопытокИнициализацииКластера = 3;
	КонецЕсли;
	Если ИнтервалПовторнойИнициализации = Неопределено Тогда
		ИнтервалПовторнойИнициализации = 1500;
	КонецЕсли;
	Если ЛогироватьЗамерыВремени = Неопределено Тогда
		ЛогироватьЗамерыВремени = Ложь;
	КонецЕсли;

КонецПроцедуры // ЗаполнитьПараметры()

// Процедура устанавливает параметры счетчиков
//
// Параметры:
//   НовыеПараметры     - Строка,     - путь к файлу настроек счетчиков
//                        Структура     или структура настроек счетчиков (см. ПараметрыСчетчиков())
//
Процедура УстановитьПараметрыСчетчиков(Знач НовыеПараметры = Неопределено) Экспорт
	
	Если ТипЗнч(НовыеПараметры) = Тип("Структура") Тогда
		ПараметрыСчетчиков = НовыеПараметры;
	ИначеЕсли ТипЗнч(НовыеПараметры) = Тип("Строка") Тогда
		ПараметрыСчетчиков = ОбщегоНазначения.ПрочитатьДанныеИзМакетаJSON(НовыеПараметры);
	Иначе
		ПараметрыСчетчиков = ОбщегоНазначения.ПрочитатьДанныеИзМакетаJSON("/config/counters", Истина);
	КонецЕсли;

КонецПроцедуры // УстановитьПараметрыСчетчиков()

// Функция - получает параметры агента кластера 1С
//
// Параметры:
//   Сервис                  - Строка        - идентификатор агента
//   НастройкиДляУстановки   - Соответствие  - настройки для чтения
//
// Возвращаемое значение:
//   Структура                          - параметры авторизации
//       *АдресСервиса    - Строка          - адрес подключения к сервису администрирования 1С
//       *ВерсияКлиента   - Строка          - версия утилиты RAC
//       *Администратор   - Строка          - имя администратора
//       *Пароль          - Строка          - пароль администратора
//       *ИБ              - Соответствие    - параметры авторизации ИБ
//
Функция ПараметрыАгента(Сервис, НастройкиДляУстановки)

	ПараметрыПоУмолчанию = ПолучитьЗначениеНастройки("__default", НастройкиДляУстановки);
	ПараметрыПоИмени     = ПолучитьЗначениеНастройки(Сервис     , НастройкиДляУстановки);

	АдресСервисаПоУмолчанию  = ПолучитьЗначениеНастройки("ras"       , ПараметрыПоУмолчанию);
	ВерсияКлиентаПоУмолчанию = ПолучитьЗначениеНастройки("rac"       , ПараметрыПоУмолчанию);
	
	АдресСервисаПоИмени  = ПолучитьЗначениеНастройки("ras"       , ПараметрыПоИмени);
	ВерсияКлиентаПоИмени = ПолучитьЗначениеНастройки("rac"       , ПараметрыПоИмени);
	Резервирует          = ПолучитьЗначениеНастройки("reserves"  , ПараметрыПоИмени, "");

	Если НЕ АдресСервисаПоИмени = Неопределено Тогда
		АдресСервиса = АдресСервисаПоИмени;
	ИначеЕсли НЕ АдресСервисаПоУмолчанию = Неопределено Тогда
		АдресСервиса = АдресСервисаПоУмолчанию;
	Иначе
		АдресСервиса      = "localhost:1545"; // RAS
	КонецЕсли;
	Если НЕ ВерсияКлиентаПоИмени = Неопределено Тогда
		ВерсияКлиента = ВерсияКлиентаПоИмени;
	ИначеЕсли НЕ ВерсияКлиентаПоУмолчанию = Неопределено Тогда
		ВерсияКлиента = ВерсияКлиентаПоУмолчанию;
	Иначе
		ВерсияКлиента     = "8.3"; // RAC
	КонецЕсли;

	ПараметрыАвторизации = ПараметрыАвторизации(Сервис, НастройкиДляУстановки);

	ПараметрыСервиса = Новый Структура();
	ПараметрыСервиса.Вставить("АдресСервиса" , АдресСервиса);
	ПараметрыСервиса.Вставить("ВерсияКлиента", ВерсияКлиента);
	ПараметрыСервиса.Вставить("Резервирует"  , Резервирует);
	ПараметрыСервиса.Вставить("Администратор", ПараметрыАвторизации.Администратор);
	ПараметрыСервиса.Вставить("Пароль"       , ПараметрыАвторизации.Пароль);

	Возврат ПараметрыСервиса;

КонецФункции // ПараметрыАгента()

// Функция - получает параметры для кластера
//
// Параметры:
//   МеткаКластера           - Строка        - ключ набора параметров
//   НастройкиДляУстановки   - Соответствие  - настройки для чтения
//
// Возвращаемое значение:
//   Структура                          - параметры авторизации
//       *Администратор   - Строка          - имя администратора
//       *Пароль          - Строка          - пароль администратора
//       *ИБ              - Соответствие    - параметры авторизации ИБ
//
Функция ПараметрыКластера(МеткаКластера, НастройкиДляУстановки)
	
	ПараметрыПоИмени     = ПолучитьЗначениеНастройки(МеткаКластера, НастройкиДляУстановки);
	НастройкиИБ          = ПолучитьЗначениеНастройки("infobase"   , ПараметрыПоИмени);

	ПараметрыАвторизации = ПараметрыАвторизации(МеткаКластера, НастройкиДляУстановки);

	ПараметрыБаз = Новый Соответствие();

	Если ТипЗнч(НастройкиИБ) = Тип("Соответствие") Тогда
		
		Для Каждого ТекНастройки Из НастройкиИБ Цикл
		
			ПараметрыИБ = ПараметрыАвторизации(ТекНастройки.Ключ, НастройкиИБ);

			Если ВРег(ТекНастройки.Ключ) = "__DEFAULT" Тогда
				КлючНастроек = "ПОУМОЛЧАНИЮ";
			Иначе
				КлючНастроек = ВРег(ТекНастройки.Ключ);
			КонецЕсли;

			ПараметрыБаз.Вставить(КлючНастроек, ПараметрыИБ);

		КонецЦикла;

	КонецЕсли;

	ПараметрыКластера = Новый Структура();
	ПараметрыКластера.Вставить("Администратор", ПараметрыАвторизации.Администратор);
	ПараметрыКластера.Вставить("Пароль"       , ПараметрыАвторизации.Пароль);
	ПараметрыКластера.Вставить("ИБ"           , ПараметрыБаз);

	Возврат ПараметрыКластера;

КонецФункции // ПараметрыКластера()

// Функция - получает параметры авторизации из переданных настроек
//
// Параметры:
//   КлючПараметров          - Строка        - ключ набора параметров
//   НастройкиДляУстановки   - Соответствие  - настройки для чтения
//
// Возвращаемое значение:
//   Структура                       - параметры авторизации
//       *Администратор   - Строка        - имя администратора
//       *Пароль          - Строка        - пароль администратора
//
Функция ПараметрыАвторизации(КлючПараметров, НастройкиДляУстановки)
	
	ПараметрыПоУмолчанию = ПолучитьЗначениеНастройки("__default", НастройкиДляУстановки);
	ПараметрыПоИмени     = ПолучитьЗначениеНастройки(КлючПараметров, НастройкиДляУстановки);

	АдминистраторПоУмолчанию = ПолучитьЗначениеНастройки("admin_name", ПараметрыПоУмолчанию);
	ПарольПоУмолчанию        = ПолучитьЗначениеНастройки("admin_pwd" , ПараметрыПоУмолчанию);
	
	АдминистраторПоИмени = ПолучитьЗначениеНастройки("admin_name", ПараметрыПоИмени);
	ПарольПоИмени        = ПолучитьЗначениеНастройки("admin_pwd" , ПараметрыПоИмени);

	Если ЗначениеЗаполнено(АдминистраторПоИмени) Тогда
		Администратор = АдминистраторПоИмени;
		Если ЗначениеЗаполнено(ПарольПоИмени) Тогда
			Пароль = ПарольПоИмени;
		КонецЕсли;
	ИначеЕсли ЗначениеЗаполнено(АдминистраторПоУмолчанию) Тогда
		Администратор = АдминистраторПоУмолчанию;
		Если ЗначениеЗаполнено(ПарольПоУмолчанию) Тогда
			Пароль = ПарольПоУмолчанию;
		КонецЕсли;
	Иначе
		Администратор = "";
		Пароль        = "";
	КонецЕсли;

	ПараметрыАвторизации = Новый Структура();
	ПараметрыАвторизации.Вставить("Администратор", Администратор);
	ПараметрыАвторизации.Вставить("Пароль"       , Пароль);

	Возврат ПараметрыАвторизации;

КонецФункции // ПараметрыАвторизации()

// Функция - получает значение настройки из соответствия или структуры по ключу без учета регистра ключа
//   
// Параметры:
//   Настройка      - Строка                    - ключ (имя) настройки
//   ВсеНастройки   - Структура, Соответствие   - контейнер настроек
//   ПоУмолчанию    - Произвольный              - значение настройки по умолчанию
//                                                возвращается если контейнер не содержит указанную настройку
//   
// Возвращаемое значение:
//    Произвольный   - значение настройки или значение по умолчанию
//
Функция ПолучитьЗначениеНастройки(Настройка, ВсеНастройки, ПоУмолчанию = Неопределено)
	
	ЭтоНастройки = (ТипЗнч(ВсеНастройки) = Тип("Соответствие")
	            ИЛИ ТипЗнч(ВсеНастройки) = Тип("ФиксированноеСоответствие")
	            ИЛИ ТипЗнч(ВсеНастройки) = Тип("Структура")
	            ИЛИ ТипЗнч(ВсеНастройки) = Тип("ФиксированнаяСтруктура"));

	Если НЕ ЭтоНастройки Тогда
		Возврат ПоУмолчанию;
	КонецЕсли;

	Для Каждого ТекНастройка Из ВсеНастройки Цикл
		Если НРег(ТекНастройка.Ключ) = НРег(Настройка) Тогда
			Возврат ТекНастройка.Значение;
		КонецЕсли;
	КонецЦикла;
	
	Возврат ПоУмолчанию;
	
КонецФункции // ПолучитьЗначениеНастройки()

#КонецОбласти // ЧтениеНастроек

#Область УстановкаПараметровОбъектовКластера

// Процедура - устанавливает параметры кластера из настроек
//
// Параметры:
//   Кластер         - Кластер        - объект кластера 1С
//
Процедура УстановитьПараметрыКластера(Кластер) Экспорт
	
	МеткаКластера = ВРег(СтрШаблон("%1:%2", Кластер.АдресСервера(), Кластер.ПортСервера()));

	ПараметрыКластера = ПараметрыПодключения.Кластеры[МеткаКластера];
	Если ПараметрыКластера = Неопределено Тогда
		ПараметрыКластера = ПараметрыПодключения.Кластеры["ПОУМОЛЧАНИЮ"];
	КонецЕсли;

	Кластер.УстановитьАдминистратора(ПараметрыКластера.Администратор, ПараметрыКластера.Пароль);

	СписокИБ = Кластер.ИнформационныеБазы().Список(, Истина);
	Для Каждого ТекИБ Из СписокИБ Цикл
		УстановитьПараметрыИБ(ТекИБ, ПараметрыКластера);
	КонецЦикла;

КонецПроцедуры // УстановитьПараметрыКластера()

// Процедура - устанавливает параметры информационной базы из настроек
//
// Параметры:
//   ИБ                  - ИнформационнаяБаза  - объект информационной базы 1С
//   ПараметрыКластера   - Соответствие        - параметры кластера
//
Процедура УстановитьПараметрыИБ(ИБ, ПараметрыКластера)
	
	Если НЕ (ПараметрыКластера.Свойство("ИБ") И ТипЗнч(ПараметрыКластера.ИБ) = Тип("Соответствие")) Тогда
		Возврат;
	КонецЕсли;

	ПараметрыИБ = ПараметрыКластера.ИБ[ВРег(ИБ.Имя())];
	Если ПараметрыИБ = Неопределено Тогда
		ПараметрыИБ = ПараметрыКластера.ИБ["ПОУМОЛЧАНИЮ"];
	КонецЕсли;

	Администратор = "";
	Пароль = "";
	Если ТипЗнч(ПараметрыИБ) = Тип("Структура") Тогда
		ПараметрыИБ.Свойство("Администратор", Администратор);
		ПараметрыИБ.Свойство("Пароль", Пароль);
	КонецЕсли;

	ИБ.УстановитьАдминистратора(Администратор, Пароль);

КонецПроцедуры // УстановитьПараметрыИБ()

#КонецОбласти // УстановкаПараметровКластера

Инициализация();