﻿// ----------------------------------------------------------
// This Source Code Form is subject to the terms of the
// Mozilla Public License, v.2.0. If a copy of the MPL
// was not distributed with this file, You can obtain one
// at http://mozilla.org/MPL/2.0/.
// ----------------------------------------------------------
// Codebase: https://github.com/ArKuznetsov/iracli/
// ----------------------------------------------------------

#Область ПрограммныйИнтерфейс

// Функция - возвращает описание доступных счетчиков для указанного типа объектов кластера
//           или всех доступных счетчиков
//
// Параметры:
//   ТипОбъектов     - Перечисление.РежимыАдминистрирования     - тип объектов кластера
//
// Возвращаемое значение:
//   Структура     - описание доступных счетчиков
//
Функция Список(Знач ТипОбъектов = Неопределено) Экспорт

	Счетчики = Новый Соответствие();

	Для Каждого ТекРаздел Из Настройки.ПараметрыСчетчиков() Цикл

		Если ЗначениеЗаполнено(ТипОбъектов) И НЕ ВРег(ТекРаздел.Ключ) = ВРег(ТипОбъектов) Тогда
			Продолжить;
		КонецЕсли;

		Счетчики.Вставить(ТекРаздел.Ключ, Новый Соответствие());
		
		РазделСчетчиков = Счетчики[ТекРаздел.Ключ];
		СчетчикиРаздела = ТекРаздел.Значение["counters"];

		Если СчетчикиРаздела = Неопределено Тогда
			Продолжить;
		КонецЕсли;

		Для Каждого ТекСчетчик Из СчетчикиРаздела Цикл
			Если НЕ ТекСчетчик.Значение["use"] Тогда
				Продолжить;
			КонецЕсли;
			РазделСчетчиков.Вставить(ТекСчетчик.Ключ, ТекСчетчик.Значение);
		КонецЦикла;

	КонецЦикла;

	Возврат Счетчики;

КонецФункции // Список()

// Функция - возвращает значения счетчиков для переданного списка объектов
//
// Параметры:
//   Объекты           - Массив из Соответствие                 - коллекция описаний объектов для получения счетчиков
//   ТипОбъектов       - Перечисления.РежимыАдминистрирования   - тип объектов кластера
//   Счетчик           - Строка                                 - имя получаемого счетчика, если не указано,
//                                                                то получаем все счетчики
//   ПервыеКоличество  - Число                                  - количество первых, максимальных значения счетчиков,
//                                                                если 0, то получаем все значения
//   Измерения         - Строка                                 - список получаемых измерений счетчика, разделенный ","
//   АгрегатнаяФункция - Строка                                 - функция свертки значений счетчиков
//                                                                (см. АгрегатныеФункции())
//   ФорматСчетчиков   - Строка                                 - формат результата (json|prometheus|plain)
//
// Возвращаемое значение:
//   Строка     - значения счетчиков
//
Функция Счетчики(Знач Объекты,
	             Знач ТипОбъектов,
	             Знач Счетчик = "",
	             Знач Измерения = "",
	             Знач АгрегатнаяФункция = "") Экспорт

	ПараметрыСчетчиковОбъекта = Настройки.ПараметрыСчетчиков()[ТипОбъектов];

    ОписаниеСчетчиков = ПараметрыСчетчиковОбъекта["counters"];

	Счетчики = ЗначенияСчетчиков(Объекты, ПараметрыСчетчиковОбъекта, Счетчик);

	Если ЗначениеЗаполнено(Измерения) Тогда
		Для Каждого ТекЭлемент Из Счетчики Цикл
            Если ЗначениеЗаполнено(АгрегатнаяФункция) Тогда
                ТекАгрегатнаяФункция = АгрегатнаяФункция;
            Иначе
                ТекАгрегатнаяФункция = ОписаниеСчетчиков[ТекЭлемент.Ключ]["aggregate"];
            КонецЕсли;
			ТекЭлемент.Значение["values"] = АгрегироватьЗначенияСчетчика(ТекЭлемент.Значение["values"],
			                                                             Измерения,
			                                                             ТекАгрегатнаяФункция);
		КонецЦикла;
	КонецЕсли;

	Для Каждого ТекЭлемент Из Счетчики Цикл
		Если НЕ ЗначениеЗаполнено(ТекЭлемент.Значение["values"]) Тогда
			ТекЭлемент.Значение["values"] = Новый Массив();
			ТекЭлемент.Значение["values"].Добавить(Новый Соответствие());
			ТекЭлемент.Значение["values"][0].Вставить("_value", Неопределено);
		КонецЕсли;
	КонецЦикла;
	
	ПрефиксСчетчиков = ПараметрыСчетчиковОбъекта["counter_prefix"];

	Возврат Новый Структура("Счетчики, Префикс", Счетчики, ПрефиксСчетчиков);

КонецФункции // Счетчики()

#КонецОбласти // ПрограммныйИнтерфейс

#Область СлужебныеПроцедурыИФункции

// Функция - выполняет свертку значений счетчиков с указанным результирующим набором измерений
// и применением указанной агрегатной функции к значениям счетчиков
//
// Параметры:
//   ЗначенияСчетчика           - Массив из Соответствие   - коллекция описаний объектов для получения счетчиков
//   РезультирующиеИзмерения    - Строка, Массив           - список измерений, попадающих в результат
//   АгрегатнаяФункция          - Строка                   - агрегатная функция (см. АгрегатныеФункции())
//
// Возвращаемое значение:
//   Массив из Соответствие                        - свернутые значения счетчиков
//     * <Имя измерения>    - Строка                 - значение измерения счетчика
//     * _value             - Число, Дата, Булево    - значение счетчика
//
Функция АгрегироватьЗначенияСчетчика(ЗначенияСчетчика,
                                     Знач РезультирующиеИзмерения = "",
                                     Знач АгрегатнаяФункция = "")

	Если ТипЗнч(РезультирующиеИзмерения) = Тип("Строка") Тогда
		Измерения = СтрРазделить(РезультирующиеИзмерения, ",", Ложь);
		Для й = 0 По Измерения.ВГраница() Цикл
			Измерения[й] = СокрЛП(Измерения[й]);
		КонецЦикла;
	ИначеЕсли ТипЗнч(РезультирующиеИзмерения) = Тип("Массив") Тогда
		Измерения = РезультирующиеИзмерения;
	Иначе
		Измерения = Новый Массив();
	КонецЕсли;

	Если НЕ Измерения.Найти("_all") = Неопределено Тогда
		Возврат ЗначенияСчетчика;
	ИначеЕсли НЕ Измерения.Найти("_no") = Неопределено Тогда
		Измерения = Новый Массив();
	КонецЕсли;

	АгрегатнаяФункция = ВРег(АгрегатнаяФункция);

	Если НЕ АгрегатныеФункции().Свойство(АгрегатнаяФункция) Тогда
		АгрегатнаяФункция = АгрегатныеФункции().Количество;
	КонецЕсли;
	
	ГруппыЗначений = Новый Соответствие();

	Для Каждого ТекЗначение Из ЗначенияСчетчика Цикл
		ИзмеренияСтрокой = "";
		Для Каждого ТекИзмерение Из Измерения Цикл
			Если ТекЗначение[ТекИзмерение] = Неопределено Тогда
				Продолжить;
			КонецЕсли;
			ИзмеренияСтрокой = СтрШаблон("%1%2=%3;", ИзмеренияСтрокой, ТекИзмерение, ТекЗначение[ТекИзмерение]);
		КонецЦикла;
		Если ГруппыЗначений[ИзмеренияСтрокой] = Неопределено Тогда
			ГруппыЗначений.Вставить(ИзмеренияСтрокой, Новый Массив());
		КонецЕсли;
		ГруппыЗначений[ИзмеренияСтрокой].Добавить(ТекЗначение);
	КонецЦикла;

	АгрегированныеЗначения = Новый Массив();

	Для Каждого ТекЭлемент Из ГруппыЗначений Цикл
		Сумма = Неопределено;
		Мин = Неопределено;
		Макс = Неопределено;
		Количество = 0;
		Различные = Новый Соответствие();

		АгрегированноеЗначение = Новый Соответствие();
		Для Каждого ТекИзмерение Из Измерения Цикл
			АгрегированноеЗначение.Вставить(ТекИзмерение, ТекЭлемент.Значение[0][ТекИзмерение]);
		КонецЦикла;
		Для Каждого ТекЗначение Из ТекЭлемент.Значение Цикл
			Если ТипЗнч(ТекЗначение["_value"]) = Тип("Число") Тогда
				Сумма = ?(Сумма = Неопределено, ТекЗначение["_value"], Сумма + ТекЗначение["_value"]);
			КонецЕсли;
			Мин = ?(Мин = Неопределено, ТекЗначение["_value"], Мин(Мин, ТекЗначение["_value"]));
			Макс = ?(Макс = Неопределено, ТекЗначение["_value"], Макс(Макс, ТекЗначение["_value"]));
			Количество = Количество + 1;
			Различные.Вставить(ТекЗначение["_value"], 1);
		КонецЦикла;
		Если АгрегатнаяФункция = АгрегатныеФункции().Сумма Тогда
			АгрегированноеЗначение.Вставить("_value", Сумма);
		ИначеЕсли АгрегатнаяФункция = АгрегатныеФункции().Минимум Тогда
			АгрегированноеЗначение.Вставить("_value", Мин);
		ИначеЕсли АгрегатнаяФункция = АгрегатныеФункции().Максимум Тогда
			АгрегированноеЗначение.Вставить("_value", Макс);
		ИначеЕсли АгрегатнаяФункция = АгрегатныеФункции().Среднее Тогда
			Если ТипЗнч(Сумма) = Тип("Число") Тогда
				АгрегированноеЗначение.Вставить("_value", Сумма / Количество);
			Иначе
				АгрегированноеЗначение.Вставить("_value", Неопределено);
			КонецЕсли;
		ИначеЕсли АгрегатнаяФункция = АгрегатныеФункции().Различные Тогда
			АгрегированноеЗначение.Вставить("_value", Различные.Количество());
		Иначе
			АгрегированноеЗначение.Вставить("_value", Количество);
		КонецЕсли;

		АгрегированныеЗначения.Добавить(АгрегированноеЗначение);
	КонецЦикла;

	Возврат АгрегированныеЗначения;

КонецФункции // АгрегироватьЗначенияСчетчика()

// Функция - получает счетчик(и) для указанного списка описаний объектов
// по указанным настройкам счетчиков
//
// Параметры:
//   Объекты                    - Массив из Соответствие        - коллекция описаний объектов для получения счетчиков
//     * <Имя поля объекта>       - Строка, Число, Дата, Булево   - значение поля объекта
//   ПараметрыСчетчиковОбъекта  - Соответствие                  - настройки получения счетчиков для типа объектов
//                                                                (см. ПараметрыСчетчиков())
//   Счетчик                   - Строка                         - имя счетчика из настроек, если не указан,
//                                                                то возвращаются все счетчики, указанные в настройках
//
// Возвращаемое значение:
//   Соответствие                                         - значения счетчиков
//     * <Имя счетчика>         - Соответствие              - содержимое счетчика
//       ** description           - Строка                    - описание счетчика из параметров
//       ** values                - Массив из Соответствие    - значения счетчика
//          *** <Имя измерения>     - Строка                    - значение измерения счетчика
//          *** _value              - Число, Дата, Булево       - значение счетчика
//
Функция ЗначенияСчетчиков(Знач Объекты, Знач ПараметрыСчетчиковОбъекта, Знач Счетчик = "")

	ОписаниеСчетчиков = ПараметрыСчетчиковОбъекта["counters"];

	ИменаСчетчиков = "";

	Счетчики = Новый Соответствие();

	Для Каждого ТекСчетчик Из ОписаниеСчетчиков Цикл
		
		Если НЕ ТекСчетчик.Значение["use"] Тогда
			Продолжить;
		КонецЕсли;

		Если НЕ ВРег(ТекСчетчик.Ключ) = ВРег(Счетчик) И ЗначениеЗаполнено(Счетчик) Тогда
			Продолжить;
		КонецЕсли;
	
		ИмяСчетчика = ТекСчетчик.Ключ;
		ИмяИсточникаСчетчика = ИмяСчетчика;
		Если НЕ ТекСчетчик.Значение["name_rac"] = Неопределено Тогда
			ИмяИсточникаСчетчика = ТекСчетчик.Значение["name_rac"];
		КонецЕсли;

		ИменаСчетчиков = ИменаСчетчиков + ?(ИменаСчетчиков = "", "", ", ") + ИмяСчетчика;

		ОписаниеСчетчика = ТекСчетчик.Значение["description"];

		Счетчики.Вставить(ИмяСчетчика, Новый Соответствие());
		Счетчики[ИмяСчетчика].Вставить("description", ОписаниеСчетчика);
		Счетчики[ИмяСчетчика].Вставить("values", Новый Массив());

		Если Объекты = Неопределено Тогда
			ЗначениеСчетчика = Новый Соответствие();
			ЗначениеСчетчика.Вставить("_value", Неопределено);
			Счетчики[ИмяСчетчика]["values"].Добавить(ЗначениеСчетчика);
			Продолжить;
		КонецЕсли;

		Для Каждого ТекОбъект Из Объекты Цикл
		
			ЗначениеСчетчика = ЗначенияИзмеренийСчетчика(ПараметрыСчетчиковОбъекта, ТекОбъект, ТекСчетчик.Значение);

			ЗначениеСчетчика.Вставить("_value", ТекОбъект[ИмяИсточникаСчетчика]);

			Счетчики[ИмяСчетчика]["values"].Добавить(ЗначениеСчетчика);

		КонецЦикла;

		Счетчики[ИмяСчетчика]["values"] = АгрегироватьЗначенияСчетчика(Счетчики[ИмяСчетчика]["values"],
		                                                               ТекСчетчик.Значение["dimentions"],
		                                                               ТекСчетчик.Значение["aggregate"]);
	КонецЦикла;
	
	Если Объекты = Неопределено Тогда
		Сообщить(СтрШаблон("Отсутствуют данные для счетчиков ""%1""", ИменаСчетчиков), СтатусСообщения.ОченьВажное);
	КонецЕсли;

	Возврат Счетчики;

КонецФункции // ЗначенияСчетчиков()

// Функция - получает значения измерений указанного описания счетчика для переданного описания объекта
// по указанным настройкам счетчиков
//
// Параметры:
//   ПараметрыСчетчиковОбъекта  - Соответствие            - настройки получения счетчиков для типа объектов
//                                                          (см. ПараметрыСчетчиков())
//   Объект                     - Соответствие            - описание объекта для получения счетчиков
//   Счетчик                    - Соответствие            - описание счетчика из настроек
//
// Возвращаемое значение:
//   Соответствие                                 - значения измерений счетчика
//     * <Имя измерения>    - Строка          - значение измерения счетчика
//
Функция ЗначенияИзмеренийСчетчика(Знач ПараметрыСчетчиковОбъекта, Знач Объект, Знач Счетчик)

	ВсеИзмерения = ПараметрыСчетчиковОбъекта["dimentions"];

	ИзмеренияСчетчика = Счетчик["dimentions"];

	ЗначенияИзмерений = Новый Соответствие();
	
	Для Каждого ТекИзмерение Из ИзмеренияСчетчика Цикл

		ОписаниеИзмерения = ВсеИзмерения[ТекИзмерение];

		Если ОписаниеИзмерения = Неопределено Тогда
			ВызватьИсключение СтрШаблон("Для счетчика ""%1"" не найдено измерение ""%2""", Счетчик["name"], ТекИзмерение);
		КонецЕсли;

		ИмяИсточникаИзмерения = ТекИзмерение;
		Если НЕ ОписаниеИзмерения["name_rac"] = Неопределено Тогда
			ИмяИсточникаИзмерения = ОписаниеИзмерения["name_rac"];
		КонецЕсли;

		ЗначениеИзмерения = Объект[ИмяИсточникаИзмерения];

		ЗначенияИзмерений.Вставить(ТекИзмерение, ЗначениеИзмерения);

	КонецЦикла;
	
	Возврат ЗначенияИзмерений;

КонецФункции // ЗначенияИзмеренийСчетчика()

// Функция - возвращает коллекцию возможных агрегатных функций
//
// Возвращаемое значение:
//   ФиксированнаяСтруктура     - возможные агрегатные функции
//
Функция АгрегатныеФункции()

	Значения = Новый Структура();
	Значения.Вставить("Сумма"     , "SUM");
	Значения.Вставить("Sum"       , "SUM");
	Значения.Вставить("Минимум"   , "MIN");
	Значения.Вставить("Min"       , "MIN");
	Значения.Вставить("Максимум"  , "MAX");
	Значения.Вставить("Max"       , "MAX");
	Значения.Вставить("Среднее"   , "AVG");
	Значения.Вставить("Avg"       , "AVG");
	Значения.Вставить("Количество", "COUNT");
	Значения.Вставить("Count"     , "COUNT");
	Значения.Вставить("Различные" , "DISTINCT");
	Значения.Вставить("Distinct"  , "DISTINCT");

	Возврат Новый ФиксированнаяСтруктура(Значения);

КонецФункции // АгрегатныеФункции()

#КонецОбласти // СлужебныеПроцедурыИФункции
